
/*
 * This file contains methods that will be used to
 * communicate instructions from the AVR processor
 * to the BasicStamp2 processor. Make sure that
 * you read and understand the documentation next
 * to each instruction before using it. 
 * If the method says "Do not use directly", then
 * don't.
 * 
 * @author Branden Archer
 */

#ifndef __BASICSTAMP2LIBRARY_H_
#define __BASICSTAMP2LIBRARY_H_

#include <avr/io.h>

#define byte char
		
/**
 * Initializes PORTA to use the BasicStamp2.
 */
void initBasicStamp();


/**
 * Sends command to robot to move so many ticks.
 * Command returning is not a guarantee that
 * distance has been traveled.
 * Range is 0-255 ticks. 1 inch is 2.3 ticks.
 *
 * @param ticks - Ticks to go forward.
 */
void forwardFast(byte ticks);

/**
 * Sends command to robot to move so many ticks.
 * Command returning is not a guarantee that
 * distance has been traveled.
 * Range is 0-255 ticks. 1 inch is 2.3 ticks.
 *
 * @param ticks - Ticks to go forward.
 */
void forwardMedium(byte ticks);

/**
 * Sends command to robot to move so many ticks.
 * Command returning is not a guarantee that
 * distance has been traveled.
 * Range is 0-255 ticks. 1 inch is 2.3 ticks.
 *
* @param ticks - Ticks to go forward.
 */
void forwardSlow(byte ticks);


/**
 * Sends command to robot to move so many ticks.
 * Command returning is not a guarantee that
 * distance has been traveled.
 * Range is 0-255 ticks. 1 inch is 2.3 ticks.
 *
 * @param ticks - Ticks to go forward.
 */
void backwardFast(byte ticks);


/**
 * Sends command to robot to move so many ticks.
 * Command returning is not a guarantee that
 * distance has been traveled.
 * Range is 0-255 ticks. 1 inch is 2.3 ticks.
 *
 * @param ticks - Ticks to go forward.
 */
void backwardMeduim(byte ticks);


/**
 * Sends command to robot to move so many ticks.
 * Command returning is not a guarantee that
 * distance has been traveled.
 * Range is 0-255 ticks. 1 inch is 2.3 ticks.
 *
 * @param ticks - Ticks to go forward.
 */
void backwardSlow(byte ticks);


/**
 * If the motor was requested to move so many
 * ticks, this replys back if those ticks have
 * been moved or not.
 *
 * @return byte - 1 if motor is done, 0 otherwise
 */
byte isMotorDone();


/**
 * Stop the motor if it is moving. Return
 * the distance it went since the last 
 * mov't command.
 *
 * @return byte - Distance motor went
 *     since last mov't command.
 */
byte stopMotor();

/**
 * If the motor was requested to move so many
 * ticks, this replys back how many of those
 * ticks have been moved.
 *
 * @return byte - number of ticks
 */
byte getTicks();


/**
 * Turns the wheels so many degrees to the right of center.
 * Ex, if want to turn toward 50 degrees on circle,
 * input 40 degrees (as 50 degrees is 40 degrees from the
 * 90 degree point).
 *
 * @param degreesFromCenter
 */
void turnRight(byte degreesFromCenter);

/**
 * Turns the wheels so many degrees to the left of center.
 * Ex, if want to turn toward 100 degrees on circle,
 * input 10 degrees (as 100 degrees is 10 degrees from the
 * 90 degree point).
 *
 * @param degreesFromCenter
 */
void turnLeft(byte degreesFromCenter);

/**
 * Sets the wheels to 90 degrees, or straight ahead.
 */
void setStraight();


/**
 * Turns off the power to the servos
 */
void turnOffSteering();

/*
 * Sets LEDs to a certain value.
 *
 * @param byte redLed
 * @param byte greenLed
 */
void setLeds(byte redLed, byte greenLed);


/*
 * Read in the value of button1
 *
 * @return byte - value of button1
 */
byte isButton1Pushed();

/*
 * Read in the value of button2
 *
 * @return byte - value of button2
 */
byte isButton2Pushed();

/*
 * Read in the value of jumper1
 *
 * @return byte - value of jumper1
 */
byte isJumper1On();

/*
 * Read in the value of jumper2
 *
 * @return byte - value of jumper2
 */
byte isJumper2On();


/*
 * Outputs a beep to the speaker of a certain
 * frequency and a certain duration.
 * Frequency: a selection of 15 predefined tones.
 * Duration range:  0-1500 ms. (input 0-15)
 *
 * @param frequency - what predefined tone
 *     to sound.
 * @param duration - how many 100's of ms
 *     Ex 1500 ms means duration = 15.
 */
void beep(byte frequency, byte duration);


/*
 * Sleep for a number of ms. There is no
 * limit to the length of delay.
 * Note: if the argument is 0, this will
 * result in a sleep of around 2ms.
 * @param delay - how many ms.
 *     Ex 500 ms means delay=500.
 */
void sleep(int delay);

/*
 * Plays the mario theme on the BasicStamp's speaker.
 * This method is non blocking. It will return before
 * the music starts playing. However, the BasicStamp 
 * will be unresponsive until the song has finished.
 * To block until the song is over, execute any other
 * command and wait for that command to return.
 */
void playMarioTheme();

/*
 * Plays the chjarge theme on the BasicStamp's speaker.
 * This method is non blocking. It will return before
 * the music starts playing. However, the BasicStamp 
 * will be unresponsive until the song has finished.
 * To block until the song is over, execute any other
 * command and wait for that command to return.
 */
void playChargeTheme();

/**
 * DO NOT USE DIRECTLY
 * Sends a command and data to the BasicStamp.
 *
 * @param command
 * @param data
 */
void send(byte command, byte data);

/**
 * DO NOT USE DIRECTLY
 * Recieves one byte of data from the BasicStamp
 *
 * @return data recieved
 */
byte recv();
		
#endif
